function [p1_medfilt,p2_medfilt,is_NaN1,is_NaN2] = filterpupil(p1,p2)

%% Define parameters used for the fixation filter

pupil_gap_margin = 100;               % Amount of data to interpolate around NaN intervals for pupil size, before and after (ms)
pupil_filt_interval = 50;             % Time interval of the moving median filter of the pupil size (ms)
median_filt_interval = 100;           % Time interval of the median filter (ms)
pupil_unusual_rate = 4000;            % If the pupil size decreases more than this amount, then the eyelid is probably closing (a.u.)
tracker_Hz = 2000;                    % Recording frequency of the eye-tracker (Hz)

p1=movmedian(p1,pupil_filt_interval,'omitmissing'); % remove high frequency jitter
p2=movmedian(p2,pupil_filt_interval,'omitmissing'); % remove high frequency jitter

p1_movmean = movmean(p1,pupil_filt_interval * (tracker_Hz/1000)); % The pupil Area is filtered using a moving mean filter.
p1_movmean_speed = tracker_Hz * diff(p1_movmean);  % Pupil size change, in Area (a.u.) per second.
unusual_pupil_change = find(p1_movmean_speed < -pupil_unusual_rate); % if the pupil size is decreasing rapidly, then this is probably a semi-blink
p1(unusual_pupil_change)=NaN; % set to NaN

p2_movmean = movmean(p2,pupil_filt_interval * (tracker_Hz/1000)); % The pupil Area is filtered using a moving mean filter.
p2_movmean_speed = tracker_Hz * diff(p2_movmean);  % Pupil size change, in Area (a.u.) per second.
unusual_pupil_change = find(p2_movmean_speed < -pupil_unusual_rate); % if the pupil size is decreasing rapidly, then this is probably a semi-blink
p2(unusual_pupil_change)=NaN; % set to NaN

p1 = 0.1169*sqrt(p1)-0.0067; % convert to mm
p2 = 0.1169*sqrt(p2)-0.0067;

%% Determine and set the gaps in the data
is_NaN1=AddMarginAroundNaN(p1,pupil_gap_margin,tracker_Hz); % Obtain NaN values after adding a margin around each NaN
is_NaN2=AddMarginAroundNaN(p2,pupil_gap_margin,tracker_Hz); % Obtain NaN values after adding a margin around each NaN

p1(is_NaN1) = NaN; % pupil data with enlarged gaps
p2(is_NaN2) = NaN; % pupil data with enlarged gaps
p1_gaps_filled = fixgaps(p1); % linearly interpolate the NaNs. Use a constant value for leading and trailing NaNs.
p2_gaps_filled = fixgaps(p2); % linearly interpolate the NaNs. Use a constant value for leading and trailing NaNs.

p1_medfilt = movmedian(p1_gaps_filled,median_filt_interval*(tracker_Hz/1000)+1);    % Apply a median filter to pupil size (add 1 to ensure the filter interval is odd, and hence symmetric)
p2_medfilt = movmedian(p2_gaps_filled,median_filt_interval*(tracker_Hz/1000)+1);    % Apply a median filter to pupil size (add 1 to ensure the filter interval is odd, and hence symmetric)
end

%% Find NaNs (blinks) in the data, and add a 'margin' number of NaNs surrounding each NaN transition. The function outputs a vector with 0s (no NaN) and 1s (NaN)
function is_NaN=AddMarginAroundNaN(y,gap_margin,tracker_Hz)

gap_margin = gap_margin * (tracker_Hz/1000); % convert gap margin from milliseconds to number of samples

is_NaN=isnan(y); % detect when there are NaNs in the eye-tracking data. NaNs may be caused by blinks.

NaN_onset_indices  = find(diff(is_NaN)==1); % indices of onset of NaN
NaN_offset_indices = 1+find(diff(is_NaN)==-1); % indices offset of NaN
NaN_switch_indices = unique([NaN_onset_indices;NaN_offset_indices]); % indexes where there is a switch from value to NaN or from NaN to value

% loop that adds an extra number of NaNs around each NaN transition. This is done because eye-tracking data quality is compromised during eyelid opening and closing.
for i=1:length(NaN_switch_indices)

    NaN_interval_with_margin = NaN_switch_indices(i) - gap_margin:NaN_switch_indices(i) + gap_margin; % interval of indices that should be set to NaN

    NaN_interval_with_margin(NaN_interval_with_margin < 1 ) = [];        % remove indexes smaller than 1
    NaN_interval_with_margin(NaN_interval_with_margin > length(y)) = []; % remove indexes larger than the data vector length

    is_NaN(NaN_interval_with_margin)=1; % set to NaN
end

end
%% Linearly interpolate missing data
function data = fixgaps(data)

if isnan(data(1)) % if the data vector starts with NaNs
    data(1:find(~isnan(data),1,'first')-1) = data(find(~isnan(data),1,'first')); % fill the NaNs with the first value in the vector
end

if isnan(data(end)) % if the data vector ends with NaNs
    data(find(~isnan(data),1,'last')+1:end) = data(find(~isnan(data),1,'last')); % fill the NaNs with the last value in the vector
end

NaN_indices=find(isnan(data)); % indices of NaNs
nNaN_indices=find(~isnan(data)); % indices of values (i.e., not NaN)

data(NaN_indices) = interp1(nNaN_indices,data(nNaN_indices),NaN_indices,'linear');

end